package org.example.country.size.action;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.example.country.size.entity.CountrySize;
import org.example.country.size.util.ServiceException;

/**
 * This is the implementation of the CountrySizeDAO interface.
 * <br>
 * <br>
 * The SQL used to query Db2 is:
 * <pre>
 * SELECT city.Name AS Capital, country.Name, country.Population, 
 *  country.SurfaceArea, country.Population / country.SurfaceArea AS Density 
 *  FROM jearl.Country AS country 
 *  INNER JOIN 
 *       jearl.City AS city  
 *  ON country.Capital = city.Id 
 *  ORDER BY country.SurfaceArea DESC
 * </pre>
 * 
 * @author Jonathan Earl
 * @see CountrySizeDAO
 *
 */
public class CountrySizeDAOImpl 
	implements CountrySizeDAO
{
	private static final Logger LOG = Logger.getLogger(CountrySizeDAOImpl.class);
	
	private Properties connectionDetails = null;
    private Connection conn = null;
    private PreparedStatement pstmt = null;
    private String sql = "SELECT city.Name AS Capital, country.Name, country.Population, "
    		+ "country.SurfaceArea, country.Population / country.SurfaceArea AS Density " 
    		+ "FROM jearl.Country AS country "
            + "INNER JOIN "
            + "     jearl.City AS city "
            + "ON country.Capital = city.Id "
            + "ORDER BY country.SurfaceArea DESC";

    /** 
     * The default constructor.
     */
    public CountrySizeDAOImpl()
    {
    	LOG.debug("Creating DAO Implementation");
    }
    
    /**
     * {@inheritDoc}
     */
	@Override
	public void connect(Properties props) 
			throws ServiceException
	{
		LOG.debug("Connecting to DB2");
        
        connectionDetails = props;
        
        String driver = connectionDetails.getProperty("db2.driver");
        
        try 
        {
            LOG.debug("Loading driver: " + driver);
            Class.forName(driver);
        } 
        catch (ClassNotFoundException e)
        {
            LOG.error("Failed to load driver");
            throw new ServiceException(e);
        }
        
        String url = connectionDetails.getProperty("db2.url");
        String user = connectionDetails.getProperty("db2.user");
        String password = connectionDetails.getProperty("db2.password");
        
        try 
        {      
        	LOG.debug("Connectiong to DB2");
        	LOG.info("Connection URL: " + url);
            conn = DriverManager.getConnection(url, user, password);
            LOG.debug("Building PreparedStatement");
            pstmt = conn.prepareStatement(sql);
        } 
        catch (SQLException e)
        {
        	LOG.error("Problem connecting to DB");
        	String errorString = getError(e);
            LOG.error(errorString);
            throw new ServiceException(errorString, e);
        }
	}

	/**
     * {@inheritDoc}
     */
	@Override
	public List<CountrySize> findCountriesBySize() 
			throws ServiceException
	{
		LOG.debug("Starting findCountriesBySize");
        List<CountrySize> results = new ArrayList<CountrySize>();
        ResultSet rset = null;
        
        try
        {
            rset = pstmt.executeQuery();
            while(rset.next())
            {
            	CountrySize record = new CountrySize();
            	record.setCapital(rset.getString("Capital"));
            	record.setName(rset.getString("Name"));
            	record.setPopulation(rset.getInt("Population"));
            	record.setArea(rset.getDouble("SurfaceArea"));
            	record.setDensity(rset.getDouble("Density"));
                results.add(record);
            }
       
            rset.close();
        }
        catch (SQLException e)
        {
            LOG.error(getError(e));
            throw new ServiceException(e);
        }
        return results;
	}

	/**
     * {@inheritDoc}
     */
	@Override
	public void disconnect()
	{
		LOG.debug("Disconnecting from DB2");
		conn = null;
	}
	
	private String getError(SQLException oops)
    {
        return "Failed to execute DB2 command," +
              " Error Code: " + oops.getErrorCode() +
              " SQLState: " + oops.getSQLState() +
              " Message: " + oops.getMessage();
    }

}
