package org.example.country.size.appl;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.example.country.size.action.CountrySizeDAO;
import org.example.country.size.action.CountrySizeDAOImpl;
import org.example.country.size.action.TextWriter;
import org.example.country.size.action.TextWriterImpl;
import org.example.country.size.entity.CountrySize;
import org.example.country.size.util.ServiceException;

/**
 * This is the entry point for the Country Size Application.
 * <br>
 * <br>
 * This application requires a property file named:
 * <strong>CountrySize.properties</strong>.
 * <br>
 * <br>
 * This property file contains the following entries:
 * <ul>
 *   <li>db2.driver</li>
 *   <li>db2.url</li>
 *   <li>db2.user</li>
 *   <li>db2.password</li>
 *   <li>file.to.write</li>
 *   <li>results.folder</li>
 * </ul>
 * 
 * @author Jonathan Earl
 * @version 1.0
 * 
 */
public class CountrySizeAppl 
{
	private static final Logger LOG = Logger.getLogger(CountrySizeAppl.class);
	
	public static void main(String[] args) 
	{
		if (LOG.isDebugEnabled())
    	{
			LOG.debug("Starting CountrySizeAppl");
		}
    	
		//  Read the properties file
        Properties props = null;
		try 
		{
			props = new Properties();
			LOG.info("Using: CountrySize.properties");
			InputStream input = new FileInputStream("CountrySize.properties");
			
			props.load(input);
		} 
		catch (IOException e) 
		{
			LOG.error("Failed to load properties");
			LOG.error(e.getMessage());
			System.exit(8);
		}
	
		CountrySizeDAO dao = new CountrySizeDAOImpl();
		try
		{
			// Connect to DB2
			LOG.debug("Connecting to DB2");
			LOG.trace("Connection URL: " + props.getProperty("db2.url"));
			dao.connect(props);
		} 
		catch (ServiceException e1)
		{
			LOG.error("Failed to connect to DB2");
			LOG.error(e1.getMessage());
			System.exit(8);
		}

		List<CountrySize> data = null;
		try
		{
			// Query the Database, returning a List<CounrtySize> objects
			LOG.debug("Querying DB2");
			data = dao.findCountriesBySize();
		} 
		catch (ServiceException e1)
		{
			LOG.error("Failed to findCountriesBySize");
			LOG.error(e1.getMessage());
			System.exit(8);
		}
		
		// Close the DB2 connection
		LOG.debug("Diconnecting from DB2");
		dao.disconnect();
		
		
		String outputFile = props.getProperty("results.folder") 
				+ File.separator + props.getProperty("file.to.write");
		LOG.info("Output file: " + outputFile);

		
		//  Create the TextWriter to handle the output
		LOG.debug("Creating TextWriter");
		TextWriter writer = new TextWriterImpl();
		try
		{
			// open the TextWriter
			LOG.debug("Opening output file");
			writer.open(outputFile);
		} 
		catch (ServiceException e)
		{
			LOG.error("Failed to open output file");
			LOG.error(e.getMessage());
			System.exit(8);
		}
			
		try
		{
			// Write the header line for the largest countries
			LOG.debug("Writing Header Line for Largest");
			writer.writeHeaderLine(TextWriter.HEADER.LARGEST);
		} 
		catch (ServiceException e)
		{
			LOG.error("Failed to writer header");
			LOG.error(e.getMessage());
			System.exit(8);
		}
      
		// Get the size of the List<CounrtySize>
		int numberOfCountries = data.size();

		int counter = 0;
		try
		{
			// loop thru the first 10 records of the List<CounrtySize>
			for (counter = 0; counter < 10; counter++)
			{
				CountrySize current = data.get(counter);
				// Format the output line
				String output = current.format();
				// Write the output line
				writer.writeLine(output);
			}
		} 
		catch (ServiceException e)
		{
			LOG.error("Failed to write output line");
			LOG.debug("Line number: " + counter);
			LOG.error(e.getMessage());
			System.exit(8);
		}
		
		try
		{
			writer.writeSpacer();
		} 
		catch (ServiceException e1)
		{
			LOG.error("Failed to write spacer");
			LOG.error(e1.getMessage());
			System.exit(8);
		}
		
		try
		{
			// Write the header line for the smallest countries
			LOG.debug("Writing Header Line for Smallest");
			writer.writeHeaderLine(TextWriter.HEADER.SMALLEST);
		} 
		catch (ServiceException e)
		{
			LOG.error("Failed to writer header");
			LOG.error(e.getMessage());
			System.exit(8);
		}
		
		// Compute the last ten records (size of the List<CountrySize> - 10)
		counter = numberOfCountries - 10;
		
		try
		{
			// loop thru the last 10 records of the List<CountrySize>
			for ( ; counter < numberOfCountries; counter++)
			{
				CountrySize current = data.get(counter);
				// Format the output line
				String output = current.format();
				// Write the output line
				writer.writeLine(output);
			}
		} 
		catch (ServiceException e)
		{
			LOG.error("Failed to write output line");
			LOG.debug("Line number: " + counter);
			LOG.error(e.getMessage());
			System.exit(8);
		}

		try
		{
			writer.close();
		}
		catch (ServiceException e)
		{
			LOG.error("Failed to close output");
			LOG.error(e.getMessage());
			System.exit(8);
		}
		
		System.exit(0);
	}
}
