package org.example.country.size.entity;

/*
 * Copyright 2021 - MFI
 *
 * Licensed under the GNU General Public License (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.gnu.org/licenses/gpl.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */


import java.io.Serializable;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.log4j.Logger;
import org.example.country.size.util.ServiceException;


/**
 * This is the Entity class to hold the data from the DB2 query.
 * 
 * 
 * The query is:
 * <pre>
 * SELECT city.Name AS Capital, country.Name, country.Population,
 *         country.SurfaceArea, country.Population / country.SurfaceArea as Density
 *  FROM jearl.Countries AS country
 *  INNER JOIN
 *       jearl.Cities AS city
 *  ON country.Capital = city.Id
 *  ORDER BY country.SurfaceArea DESC
 * </pre>
 * 
 * @author Jonathan Earl
 * @since Version 1.0
 *
 */
public class CountrySize
	implements Serializable
{
	private static final Logger LOG = Logger.getLogger(CountrySize.class);
	private static final long serialVersionUID = 1L;

	private String capital;
	private String name;
	private int population;
	private double area;
	private double density;
	
	/**
	 * The default constructor of CountrySize.
	 * 
	 * The initial values are:
	 * <ul>
	 *   <li>Capital: <strong>Birnin Zana</strong></li>
	 *   <li>Name: <strong>Wakanda</strong></li>
	 *   <li>Population: <strong>6000000</strong></li>
	 *   <li>Area: <strong>22384.0</strong></li>
	 *   <li>Density: <strong>268.05</strong></li>
	 * </ul>
	 * 
	 * @throws ServiceException
	 *         if the CountrySize object cannot be built
	 */
	public CountrySize() 
			throws ServiceException
	{
		LOG.debug("Constructing default CountrySize object");
		setCapital("Birnin Zana");
		setName("Wakanda");
		setPopulation(6000000);
		setArea(22384.0);
		setDensity(getPopulation() / getArea());
	}
	
	/**
	 * Returns the capital.
	 * 
	 * @return the capital
	 */
	public String getCapital()
	{
		LOG.debug("Returning capital of: " + capital);
		return capital;
	}
	
	/**
	 * Sets the capital field.
	 * 
	 * The capital requirements:
	 * <ul>
	 *   <li>Must not be null</li>
	 *   <li>Must not be empty</li>
	 *   <li>Must 35 char or less in length</li>
	 * </ul>
	 * 
	 * @param capital the value to set
	 * @throws ServiceException 
	 *         if the capital does not pass the requirements listed above 
	 */
	public void setCapital(String capital) 
			throws ServiceException
	{
		if (capital == null)
		{
			throw new ServiceException("Capital must not be null");
		}
		capital = capital.trim();
		if (capital.isEmpty())
		{
			throw new ServiceException("Capital must not be empty");
		}
		if (capital.length() > 35)
		{
			throw new ServiceException("Capital name must be 35 chars or less in length");
		}
		
		this.capital = capital;
	}
	
	/**
	 * Returns the name.
	 * 
	 * @return the name
	 */
	public String getName()
	{
		LOG.debug("Returning name of: " + name);
		return name;
	}
	
	/**
	 * Sets the name field.
	 * 
	 * The name requirements:
	 * <ul>
	 *   <li>Must not be null</li>
	 *   <li>Must not be empty</li>
	 * </ul>
	 * 
	 * @param name the value to set
	 * @throws ServiceException
	 *         if the name does not pass the requirements listed above  
	 */
	public void setName(String name) 
			throws ServiceException
	{
		if (name == null)
		{
			throw new ServiceException("Name must not be null");
		}
		name = name.trim();
		if (name.isEmpty())
		{
			throw new ServiceException("Name must not be empty");
		}
		this.name = name;
	}
	
	/**
	 * Returns the population.
	 * 
	 * @return the population
	 */
	public int getPopulation()
	{
		LOG.debug("Returning population of: " + population);
		return population;
	}
	
	/**
	 * Sets the population field.
	 * 
	 * The population requirements:
	 * <ul>
	 *   <li>Must greater than zero</li>
	 * </ul>
	 * @param population the value to set
	 * @throws ServiceException
	 *         if the population does not pass the requirements listed above  
	 */
	public void setPopulation(int population) 
			throws ServiceException
	{
		if (population < 1)
		{
			throw new ServiceException("Population must be greater than zero");
		}
		this.population = population;
	}
	
	/**
	 * Returns the area.
	 * 
	 * @return the area
	 */
	public double getArea()
	{
		LOG.debug("Returning area of: " + area);
		return area;
	}
	
	/**
	 * Sets the area field.
	 * 
	 * The area requirements:
	 * <ul>
	 *   <li>Must greater than zero</li>
	 * </ul>
	 * @param area the value to set
	 * @throws ServiceException
	 *         if the area does not pass the requirements listed above  
	 */
	public void setArea(double area) 
			throws ServiceException
	{
		if (area <= 0)
		{
			throw new ServiceException("Area must be greater than zero");
		}
		this.area = area;
	}
	
	/**
	 * Returns the density.
	 * 
	 * @return the density
	 */
	public double getDensity()
	{
		LOG.debug("Returning density of: " + density);
		return density;
	}
	
	/**
	 * Sets the density field.
	 * 
	 * The density requirements:
	 * <ul>
	 *   <li>Must greater than zero</li>
	 * </ul>
	 * @param density the value to set
	 * @throws ServiceException
	 *         if the density does not pass the requirements listed above  
	 */
	public void setDensity(double density) 
			throws ServiceException
	{
		if (density <= 0)
		{
			throw new ServiceException("Density must be greater than zero");
		}
		this.density = density;
	}
	
	/**
	 * This formats the record for print.
	 * 
	 * @return a formatted string
	 */
	public String format()
	{
		String output = " %-21s    %-30s     %,13d  %,12.1f  %,10.2f";
		return String.format(output,  
				getCapital(), getName(),  
				getPopulation(), getArea(),
				getDensity());
	}

	/**
	 * Customizes the toString.
	 * 
	 * @return the customized string
	 */
	@Override
	public String toString()
	{
		return "CountrySize [capital=" + capital + ", name=" + name
				+ ", population=" + population + ", area=" + area
				+ ", density=" + density + "]";
	}
	
	/**
	 * Uses all fields to create a hash code.
	 * 
	 */
	@Override
	public int hashCode(){
		return new HashCodeBuilder()
		.append(capital)
		.append(name)
		.append(population)
		.append(area)
		.append(density)
		.toHashCode();
	}	


	/**
	 * Uses all fields to test for equality.
	 * 
	 */
	@Override
	public boolean equals(final Object obj) {
		if(obj instanceof CountrySize) {
			final CountrySize other = (CountrySize) obj;
			return new EqualsBuilder()
			.append(capital, other.capital)
			.append(name, other.name)
			.append(population, other.population)
			.append(area, other.area)
			.append(density, other.density)
			.isEquals();
		} else {
			return false;
		}
	}
}
